#ifndef __LINKED_LIST_CPP__
#define __LINKED_LIST_CPP__

#include "LinkedList.h"

// stdlib defines the symbol NULL
#include <stdlib.h>

/**
 * Initialize _front to NULL and set the length to 0
 */
template <class T>
LinkedList<T>::LinkedList() : _front(NULL), _length(0) {

}

/**
 * Copies the elements of l to this object
 * using the copyElements function.  Note that
 * the copyElements function also sets the length
 * variable.
 */
template <class T>
LinkedList<T>::LinkedList(const LinkedList<T>& l) {
	copyElements(l._front);
}


/**
 * Copies the elements of node onward to this object
 */
template <class T>
void LinkedList<T>::copyElements(Node<T>* node) {
	// If our list of nodes isn't empty, let's start the copy
	if (node != NULL) {
		// make sure we get our _front value set correctly
		_front = new Node<T>(node->_value);
		// make a temp variable to iterate thru our copy
		Node<T>* temp = _front;
		// set our length to 1
		_length = 1;
		// move to the next node in the original
		node = node->_next;
		while (node != NULL) {
			// make our copy
			temp->_next = new Node<T>(node->_value);
			// iterate over the copy
			temp = temp->_next;
			// iterate over the original
			node = node->_next;
			// update our length
			_length = _length + 1; 
		}
	}
}

/**
 * Deletes all the elements from _front onward
 */
template <class T>
void LinkedList<T>::deleteElements() {
	while (_length != 0) {
		Node<T>* temp = _front->_next;
		delete _front;
		_front = temp;
		_length = _length - 1;
	}	
}

/**
 * Assignment operator
 */
template <class T>
LinkedList<T>& LinkedList<T>::operator=(const LinkedList<T>& l) {
	deleteElements();
	copyElements(l._front);
	return(*this);
}

/**
 * Destructor
 */
template <class T>
LinkedList<T>::~LinkedList() {
	deleteElements();
}

/**
 * Adds an element to the front of the list
 */
template <class T>
void LinkedList<T>::add(T value) {
	Node<T>* temp = _front;
	_front = new Node<T>(value);
	_front->_next = temp;
	++_length;
}

/**
 * Removes the front element of the list
 */
template <class T>
T LinkedList<T>::remove() {
	if (!empty()) {
		Node<T>* temp = _front;
		_front = _front->_next;
		delete temp;
		--_length;
	}	
}

/**
 * Returns the front value of the list without
 * altering the list.
 */
template <class T>
T LinkedList<T>::front() const {
	return(_front->_value);
}

/**
 * Am I empty?
 */
template <class T>
bool LinkedList<T>::empty() const {
	return(_length == 0);
}	

/**
 * Returns the size of the list
 */
template <class T>
long LinkedList<T>::size() const {
	return(_length);
}

/**
 * Tells if two lists are equal or not.
 * Note that even though you may have altered the function
 * prototype in the header file, it is unncessary to do so here.
 */
template <class T>
bool operator==(const LinkedList<T>& l1, const LinkedList<T>& l2) {
	if (l1.size() == l2.size()) {
		int counter = 0;
		Node<T>* n1 = l1._front;
		Node<T>* n2 = l2._front;
		for (int i = 0; i < l1.size(); ++i) {
			if (n1->_value != n2->_value) {
				return(false);
			} else {
				n1 = n1->_next;
				n2 = n2->_next;
			}
		}
		return(true);
	} else {
		return(false);
	}
}
		


#endif // __LINKED_LIST_CPP__