#ifndef __STACK_H__
#define __STACK_H__

// when using a parameterized class, you must
// include the implementation file
#include "LinkedList.cc"

/**
 * Read LinkedList.h for an explanation about why
 * we need to declare the class here, the function below it,
 * and then its redeclaration.
 */
template <class T> class Stack;

template <class T>
bool operator==(const Stack<T>& s1, const Stack<T>& s2);

/**
 * A stack is a First-In, Last-out data structure.
 * I think one of you likened it to a PEZ dispenser.  When
 * you insert an object into a stack, it pushes all the other
 * elements down.  When you remove an element, you pop the first element
 * off.
 */
template <class T> class Stack {

	private:
		/**
		 * We will use the list as our underlying data structure
		 * for the stack.  Hence, all the functions below will operate
		 * on this _list object
		 */
		LinkedList<T> _list;

	public:
		/**
		 * Default constructor
		 * 
		 * Remember that _list is already initialized, so the
		 * implementation of this function doesn't need to do anything
		 */
		Stack();

		/**
		 * The copy constructor only really needs to copy
		 * the list elements over.
		 */
		Stack(const Stack<T>& s);
		
		/**
		 * The assignment operator deletes its current
		 * elements and then copies the elements of s over
		 */
		Stack<T>& operator=(const Stack<T>& s);
		
		/**
		 * This operation removes one element from the stack
		 */
		T pop();
		
		/**
		 * This operation adds one element onto the stack
		 */
		void push(T value);
		
		/**
		 * This operator returns the top element of the stack
		 * without removing it.
		 */
		T top() const;
		
		/**
		 * This tells us if the stack is empty or not
		 */
		bool empty() const;


		// OIT's g++ does not like this declaration because it is an old
		// compiler. If you are using OIT's g++ you should replace
		// the declaration below with:
		//
		// friend bool operator==(const Stack<T>& s1, const Stack<T>& s2);
		//
		// The implementation should still look like:
		//
		// template <class T>
		// bool operator==(const Stack<T>& s1, const Stack<T>& s2) {
		//   ...
		// }
		//
		//  If you're not using OIT's g++ and your compiler is 
		//  complaining about not knowing about the
		//  operator== function, than try removing the '<>' right
		//  after the operator== so that the line looks like the one
		//  above.
		//
		//  Two stacks are equal if all their elements are equal
		friend bool operator==<>(const Stack<T>& s1, const Stack<T>&s2);


};

#endif // __STACK_H__
